/*******************************************************************
 *
 * Name:	TSAVPluginMotif.c
 *
 * Description: Audio-video plugin - H.263 + GSM
 *
 * Author:	Tomasz Stachowiak
 *
 * Date:	15/08/96
 *
 * Change Log:
 *
 ********************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <netdb.h>
#include <netinet/in.h>

#include <Xm/Xm.h>	    /* Motif includes */
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/MainW.h>

#include <dve/dve.h>	    /* DVE includes */
#include <dve/confMgr.h>

#include "TSAudioPluginMotif.h"
#include "AudioPluginProtocol.h"

#define XT

extern DveSession dveSession;	/* global variables */
extern XtAppContext app;
extern int current_state;
extern XtWorkProcId workproc;

void SendAVFrame();

void
QuitApplication()
{
   gsm_close();
   shutdownEncoder();
   CleanupPlugin();
   exit(0);
}

/*
 * Main()
 */
void
main(int argc, char **argv)
{

  Widget toplevel, toprowcol, rowcol, start_but, stop_but, scale,
	 main_w, menubar;
  void start_pushed();
  void stop_pushed();
  void scale_change();
  void file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs);
  XmString label, str1, str2;



  /*
   * Speaker and state initialization
   */
  InitSpeaker();
  current_state = STOP_STATE;

  gsm_init();                      /* gsm codec initialization */

  initcapture(176,144);            /* image capturing initialization */
  initEncoder(176, 144, 100000);   /* h.263 encoder init */

  /*
   * Motif interface initialization
   */

/* Application */
  toplevel = XtVaAppInitialize(&app, "Audio plugin", NULL, 0, 
			       &argc, argv, NULL,	   
			       NULL);
  
/* Main window */
  main_w =  XtVaCreateManagedWidget("main_w", xmMainWindowWidgetClass, 
				    toplevel,
				    NULL);
/* Menu bar */
  str1 = XmStringCreateSimple("File");
  menubar = XmVaCreateSimpleMenuBar(main_w, "menubar",
	    XmVaCASCADEBUTTON, str1, 'F',
	    NULL);
  XmStringFree(str1);
				    
/* Pulldown menu (File) */
  str1 = XmStringCreateSimple("Exit");
  str2 = XmStringCreateSimple("Ctrl+C");
  XmVaCreateSimplePulldownMenu(menubar, "file_menu", 0, file_menu,
	    XmVaPUSHBUTTON, str1, 'E', "Ctrl<Key>c", str2,
	    NULL);
  XmStringFree(str1);
  XmStringFree(str2);
  

/* Vertical rowcolum widget */
  toprowcol =  XtVaCreateManagedWidget("toprowcol", xmRowColumnWidgetClass, 
				      main_w,
				      XmNorientation, XmVERTICAL,
				      NULL);

/* Horizontal rowcolumn widget (for buttons) */
  rowcol =  XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
				      toprowcol,
				      XmNorientation, XmHORIZONTAL,
				      NULL);
/* Start button */  
  label = XmStringCreateSimple("START");
  start_but = XtVaCreateManagedWidget("start", xmPushButtonWidgetClass, rowcol,
				      XmNwidth, 100,
				      XmNalignment, XmALIGNMENT_CENTER,
				      XmNlabelString, label,
				      NULL);
  XmStringFree(label);

/* Stop button */
  label = XmStringCreateSimple("STOP");
  stop_but = XtVaCreateManagedWidget("stop", xmPushButtonWidgetClass, rowcol,
				     XmNlabelString, label,
				     NULL);
  XmStringFree(label);

/* Buttons callbacks setting */
  XtAddCallback(start_but, XmNactivateCallback, start_pushed, NULL);
  XtAddCallback(stop_but, XmNactivateCallback, stop_pushed, NULL);

  XtManageChild(rowcol);

/* Volume scale */
  scale = XtVaCreateManagedWidget("scale", xmScaleWidgetClass, toprowcol,      
		XtVaTypedArg, XmNtitleString, XmRString, "Volume", 6,
		XmNorientation, XmHORIZONTAL,
	        XmNmaximum, 100,
		XmNminimum, 0,
		XmNvalue, GetVolume(),
		XmNshowValue, True,
		NULL);

/* Volume scale callback setting */
  XtAddCallback(scale, XmNvalueChangedCallback, scale_change, NULL);

  XtManageChild(menubar);
  XtRealizeWidget(toplevel);



   /*
    * Start up OpenDVE
    */
   if (InitializeOpenDVE(app, AUDIO_PLUGIN_PORT)==NULL)
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }

  XtAppMainLoop(app);
}


/*
 * Buttons pushing callbacks
 */

void start_pushed()
{
  if (current_state == STOP_STATE)
    {
      current_state = START_STATE;
      InitMic();
      workproc = XtAppAddWorkProc(app, SendAVFrame, NULL);
    }
}

void stop_pushed()
{
  if (current_state == START_STATE)
  {
    current_state = STOP_STATE;
    XtRemoveWorkProc(workproc);
    CloseMic();
  }
}

/*
 * Scale callback
 */
void
scale_change(w, client_data, cbs)
Widget w;
caddr_t client_data;
XmScaleCallbackStruct *cbs;
{
  VolumeChange(cbs->value);
}

/*
 * File menu callback
 */
void file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
  if(item_no == 0)
    QuitApplication();
}













